/**
 * Content script injected into current DOM tree.
 */

"use strict";

const searchInputSelector = '#gbqfq,#yschsp,#p_13838465-p,#sb_form_q';
const injectPanelSelector = '#center_col,#results_container,#cols';
const injectPanelSelectorBing = '#results_container > div.sb_ph';

const googleRegExp = /https?:\/\/(www\.)?google.(.*?)/i;
const yahooRegExp = /https?:\/\/(.*?).search.yahoo.com/i;
const bingRegExp = /https?:\/\/(www\.)?bing.com/i;

var searchTerm = null;
var addon = null;


if (unsafeWindow.self == unsafeWindow.top) {

  if (googleRegExp.test(document.location.href) ||
      yahooRegExp.test(document.location.href) ||
      bingRegExp.test(document.location.href)) {
    setInterval(checkSearch, 2500);
  }

  if (!addon) {
    self.port.emit('get-addon');
  }

}

self.port.on('set-addon', function (a) {
  addon = a;

  checkSearch();
  checkPromos();
  checkStores();
  checkHotels();
  checkFlights();
  checkRentals();
  checkEngines();
});

self.port.on('set-search-response', function (responseText) {
  injectPanel(responseText);
});

self.port.on('set-coupon-response', function (responseText) {
  injectPanel(responseText);
});

self.port.on('set-promo-response', function (responseText) {
  injectTop(responseText);
});

self.port.on('set-store-response', function (responseText) {
  injectTop(responseText);
});

self.port.on('set-hotel-response', function (responseText) {
  injectTop(responseText);
});

self.port.on('set-flight-response', function (responseText) {
  injectTop(responseText);
});

self.port.on('set-rental-response', function (responseText) {
  injectTop(responseText);
});

self.port.on('set-engine-response', function (responseText) {
  injectTop(responseText);
});

function checkRentals() {
  var url = document.location.href;
  log('checkRentals: url = ' + url);
  var domain = getDomain();
  log('checkRentals: domain = ' + domain);
  
  for (var i = 0; i < addon.rentals.length; i++) {
    var rental = addon.rentals[i];
    log('checkRentals: regexp = ' + rental.url);
    var regex = new RegExp(rental.url, "i");
    if (regex.test(url)) {
        log('checkRentals: url matched');
        var from = getDataValue(rental, "from");
        log('checkRentals: from = ' + from);
        var to = getDataValue(rental, "to");
        log('checkRentals: to = ' + to);
        var dpday = getDataValue(rental, "dpday");
        log('checkRentals: dpday = ' + dpday);
        var dpmonth = getDataValue(rental, "dpmonth");
        log('checkRentals: dpmonth = ' + dpmonth);
        var dpyear = getDataValue(rental, "dpyear");
        log('checkRentals: dpyear = ' + dpyear);
        var rtday = getDataValue(rental, "rtday");
        log('checkRentals: rtday = ' + rtday);
        var rtmonth = getDataValue(rental, "rtmonth");
        log('checkRentals: rtmonth = ' + rtmonth);
        var rtyear = getDataValue(rental, "rtyear");
        log('checkRentals: rtyear = ' + rtyear);
        var age = getDataValue(rental, "age");
        log('checkRentals: age = ' + age);
        var dpdate = convertDate(getDataValue(rental, "dpdate"));
        log('checkRentals: dpdate = ' + dpdate);
        var rtdate = convertDate(getDataValue(rental, "rtdate"));
        log('checkRentals: rtdate = ' + rtdate);
        var serverUrl = 'http://xml.gocheapr.com/rental.php?url=' + domain +
                          '&from=' + from +
                          '&to=' + to +
                          '&dpdate=' + dpdate +
                          '&rtdate=' + rtdate +
                          '&dpday=' + dpday +
                          '&dpmonth=' + dpmonth +
                          '&dpyear=' + dpyear +
                          '&rtday=' + rtday +
                          '&rtmonth=' + rtmonth +
                          '&rtyear=' + rtyear +
                          '&age=' + age;
        log('checkRentals: server url = ' + serverUrl);
        self.port.emit('send-hotel-request', serverUrl);
        return true;
    }
  }

  log('checkRentals: no match');
  return false;
}

function checkFlights() {
  var url = document.location.href;
  log('checkFlights: url = ' + url);
  var domain = getDomain();
  log('checkFlights: domain = ' + domain);
  
  for (var i = 0; i < addon.flights.length; i++) {
    var flight = addon.flights[i];
    log('checkFlights: regexp = ' + flight.url);
    var regex = new RegExp(flight.url, "i");
    if (regex.test(url)) {
        log('checkFlights: url matched');
        var from = getDataValue(flight, "from");
        log('checkFlight: from = ' + from);
        var to = getDataValue(flight, "to");
        log('checkFlight: to = ' + to);
        var dpdate = convertDate(getDataValue(flight, "dpdate"));
        log('checkFlight: dpdate = ' + dpdate);
        var rtdate = convertDate(getDataValue(flight, "rtdate"));
        log('checkFlight: rtdate = ' + rtdate);
        var adult = getDataValue(flight, "adult");
        log('checkFlight: adult = ' + adult);
        var children = getDataValue(flight, "children");
        log('checkFlight: children = ' + children);
        var infant = getDataValue(flight, "infant");
        log('checkFlight: infant = ' + infant);
        var senior = getDataValue(flight, "senior");
        log('checkFlight: senior = ' + senior);
        var serverUrl = 'http://xml.gocheapr.com/flight.php?url=' + domain +
                          '&from=' + from +
                          '&to=' + to +
                          '&dpdate=' + dpdate +
                          '&rtdate=' + rtdate +
                          '&adult=' + adult +
                          '&children=' + children +
                          '&infant=' + infant +
                          '&senior=' + senior;
        log('checkFlights: server url = ' + serverUrl);
        self.port.emit('send-flight-request', serverUrl);
        return true;
    }
  }

  log('checkFlights: no match');
  return false;
}

function checkHotels() {
  var url = document.location.href;
  log('checkHotels: url = ' + url);
  var domain = getDomain();
  log('checkHotels: domain = ' + domain);
  
  for (var i = 0; i < addon.hotels.length; i++) {
    var hotel = addon.hotels[i];
    log('checkHotels: regexp = ' + hotel.url);
    var regex = new RegExp(hotel.url, "i");    
    if (regex.test(url)) {
      log('checkHotels: url matched');
      var dest = getDataValue(hotel, "dest");
      log('checkHotels: dest = ' + dest);
      var checkin = convertDate(getDataValue(hotel, "checkin"));
      log('checkHotels: checkin = ' + checkin);
      var checkout = convertDate(getDataValue(hotel, "checkout"));
      log('checkHotels: checkout = ' + checkout);
      var hotelV = getDataValue(hotel, "hotel");
      log('checkHotels: hotel = ' + hotelV);
      var adult = getDataValue(hotel, "adult");
      log('checkHotels: adult = ' + adult);
      var children = getDataValue(hotel, "children");
      log('checkHotels: children = ' + children);
      var age = getDataValue(hotel, "age");
      log('checkHotels: age = ' + age);
      var currency = getDataValue(hotel, "currency");
      log('checkHotels: currency = ' + currency);
      var lang = getDataValue(hotel, "lang");
      log('checkHotels: lang = ' + lang);
      var serverUrl = 'http://xml.gocheapr.com/hotels.php?url=' + domain +
                        '&dest=' + dest +
                        '&checkin=' + checkin +
                        '&checkout=' + checkout +
                        '&hotel=' + hotelV +
                        '&adult=' + adult +
                        '&children=' + children +
                        '&age=' + age +
                        '&currency=' + currency +
                        '&lang=' + lang;
      log('checkHotels: server url = ' + serverUrl);
      self.port.emit('send-hotel-request', serverUrl);
      return true;
    }
  }

  log('checkHotels: no match');
  return false;
}

function checkStores() {
  var url = document.location.href;
  log('checkStores: url = ' + url);
  var domain = getDomain();
  log('checkStores: domain = ' + domain);

  for (var i = 0; i < addon.stores.length; i++) {
    var store = addon.stores[i];
    log('checkStores: regexp = ' + store.url);
    var regex = new RegExp(store.url, "i");
    if (regex.test(url)) {
        log('checkStores: url matched');
        var title = getDataValue(store, "title");
        log('checkStores: title = ' + title);
        var price = getDataValue(store, "price");
        log('checkStores: price = ' + price);
        var upc = getDataValue(store, "upc");
        log('checkStores: upc = ' + upc);
        var serverUrl = 'http://xml.gocheapr.com/stores.php?feed=' + store.feed +
                                                          '&url=' + domain +
                                                          '&title=' + encodeURI(title) +
                                                          '&price=' + encodeURI(price) +
                                                          '&upc=' + encodeURI(upc);
        log('checkStores: server url = ' + serverUrl);
        self.port.emit('send-store-request', serverUrl);
        return true;
    }
  }

  log('checkStores: no match');
  return false;
}

function checkPromos() {
  var domain = getDomain();
  log('checkPromos: domain = ' + domain);
  log('checkPromos: promos.length = ' + addon.promos.length);

  for (var i = 0; i < addon.promos.length; i++) {
    var promo = addon.promos[i];
    if (promo.url == domain) {
      var url = 'http://xml.gocheapr.com/promos.php?url=' + domain;
      log('checkPromos: server url = ' + url);
      self.port.emit('send-promo-request', url);
      return true;
    }
  }

  log('checkPromos: no match');
  return false;
}

function checkEngines() {
  var url = document.location.href;
  log('checkEngines: url = ' + url);
  var domain = getDomain();
  log('checkEngines: domain = ' + domain);

  for (var i = 0; i < addon.engines.length; i++) {
    var engine = addon.engines[i];
    log('checkEngines: regexp = ' + engine.url);
    var regex = new RegExp(engine.url, "i");
    if (regex.test(url)) {
        log('checkEngines: url matched');
    	var keyword1 = url.match(regex);
		var keyword = keyword1[1];
        log('checkEngines: keyword = ' + keyword );
        var serverUrl = 'http://xml.gocheapr.com/engines.php?cat=' + engine.cat +
                                                          '&url=' + domain +
                                                          '&keyword=' + encodeURI(keyword);
        log('checkEngines: server url = ' + serverUrl);
        self.port.emit('send-engine-request', serverUrl);
        return true;
    }
  }

  log('checkEngines: no match');
  return false;
}

function injectTop(html) {
  $('body').prepend(html);
}

function injectPanel(html) {
  removePanel();
  
  var $panel = $('<div id="gocheapr_panel" style="padding:3px"></div>');
  $panel.append(html);

  if ($('#gocheapr_panel').length == 0) {
    if ($(injectPanelSelectorBing).length > 0) {
      $(injectPanelSelectorBing).after($panel);
    } else {
      $(injectPanelSelector).prepend($panel);
    }
  }

  log('injectPanel: gocheapr panel injected');
}

function removePanel() {
  $('#gocheapr_panel').remove();
}

function checkSearch() {
  var serp = isSERP();
  if (serp) {
    var searchTermNew = $(searchInputSelector).val();
    log('checkSearch: input value = ' + searchTermNew);
    if (searchTermNew != searchTerm) {
      searchTerm = searchTermNew;
      log('checkSearch: searchTerm = ' + searchTerm);
      if (matchSearchTerm()) {
        var domain = getDomain();
        var url = 'http://xml.gocheapr.com/search.php?q=' + searchTerm + '&site=' + domain;
        log('checkSearch: server url = ' + url);
        self.port.emit('send-search-request', url);
      } else if(matchCoupon()) {
        var domain = getDomain();
        var url = 'http://xml.gocheapr.com/coupon.php?q=' + searchTerm + '&site=' + domain;
        log('checkSearch: server url = ' + url);
        self.port.emit('send-coupon-request', url);
      } else {
        log('checkSearch: keyword not matched');
        removePanel();
      }
    }
  }
}

function matchSearchTerm() {
  log('matchSearchTerm: keywords.length = ' + addon.keywords.length);

  for (var i = 0; i < addon.keywords.length; i++) {
    var keyword = addon.keywords[i];
    if (keyword.q == searchTerm) {
        log('matchSearchTerm: matched => ' + searchTerm);
        return true;
    }
  }

  log('matchSearchTerm: no match');
  return false;
}

function matchCoupon() {
  log('matchCoupon: coupons.length = ' + addon.coupons.length);

  for (var i = 0; i < addon.coupons.length; i++) {
    var keyword = addon.coupons[i];
    if (keyword.q == searchTerm ||
        keyword.q + " coupon" == searchTerm ||
        keyword.q + " coupons" == searchTerm) {
      log('matchCoupon: matched q => ' + searchTerm);
      return true;
    }
    if (keyword.domain == searchTerm ||
        keyword.domain + " coupon" == searchTerm ||
        keyword.domain + " coupons" == searchTerm) {
      log('matchCoupon: matched domain => ' + searchTerm);
      return true;
    }
  }

  log('matchCoupon: no match');
  return false;
}

function isSERP() {
  var $resultDiv = $(injectPanelSelector);
  if ($resultDiv.length > 0) {
    log('isSERP: SERP page found');
    return true;
  } else {
    log('isSERP: not on SERP page');
    return false;
  }
}

function getDomain() {
    var url = document.location.href;
    log('getDomain: url = ' + url);
    var re = new RegExp('^(?:f|ht)tp(?:s)?\://([^/]+)', 'im');
    var domain = url.match(re)[1].toString().replace(/www./i, '');
    log('getDomain: domain = ' + domain);
    return domain;
}

function getUrlVars() {
  var vars = [], hash;
  var hashes = document.location.href.slice(window.location.href.indexOf('?') + 1).split('&');
  for(var i = 0; i < hashes.length; i++) {
    hash = hashes[i].split('=');
    vars.push(hash[0]);
    vars[hash[0]] = hash[1];
  }
  return vars;
}

function getUrlVar(name) {
  return getUrlVars()[name];
}

function getParseValue(selector) {
  if ($(selector).is('input,select')) {
    return $(selector).val();
  } else {
    return $(selector).text();
  }
}

function getDataValue(root, name) {
  var data = '';
  
  if (root.fromurl) {
    data = getUrlVar(root.fromurl[name]);
  }
  
  if (isEmpty(data) && root.parse) {
    data = getParseValue(root.parse[name]);
  }
  
  if (!data) {
    data = '';
  }
  
  return data;
}

function isEmpty(str) {
  return (!str || 0 === str.length);
}

function convertDate(dateStr) {  
  log('convertDate: before = ' + dateStr);
  if (!dateStr) {
    return '';
  }
  var date = Date.parse(dateStr);
  var converted = date.toString('yyyy-MM-dd');
  log('convertDate: after = ' + converted);
  return converted;
}

function log(message) {
  console.log('GoCheapR > ' + message);
}
